function widget:GetInfo()
  return {
    name      = "FPSManager",
    desc      = "Tries to keep the framerate around 30 by adjusting the detail level.",
    author    = "quantum",
    date      = "May 28, 2007",
    license   = "GNU GPL, v2 or later",
    layer     = 0,
    enabled   = false  --  loaded by default?
  }
end

--------------------------------------------------------------------------------
----------------------------Configuration---------------------------------------
local maxFps     = 40
local minFps     = 20
local tolerance  = 10
------------     increase detail  ----  reduce detail     ----------------------
--------------------------------------------------------------------------------
levelTable = 
{ 
               {"advshading 1"          , "advshading 0"      }, --level 1, enable reflective units & transparency
               {"maxparticles 10000"    , "maxparticles 5000" }, --level 2, increase max particles to 10000
               {"water 3"               , "water 0"           }, --level 3, set water type to reflective + refractive
               {"shadows 1 1024"        , "shadows 0"         }, --level 4, enable shadows, shadow map size set to 1024
               {"water 4"               , "water 3"           }, --level 5, set water type to bump mapped
               {"maxparticles 20000"    , "maxparticles 10000"}, --level 6, increase max particles to 20000
               {"water 2"               , "water 4"           }, --level 7, set water type to dynamic
               {"shadows 1 2048"        , "shadows 1 1024"    }, --level 8, increase shadow map size to 2048
               {"dynamicsky 1"        	, "dynamicsky 0"      }, --level 9, enable dynamic sky
               {"shadows 1 4096"        , "shadows 1 2048"    }, --level 10,increase shadow map size to 4096
}

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


local SendCommands  = Spring.SendCommands
local counter       = 0
local detailLevel   = 4
local fpsTable      = {}
local raisePoints   = 0
local lowerPoints   = 0


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


local function RaiseDetail()
  raisePoints = raisePoints + 1
  if ((detailLevel <= #levelTable) and
      (raisePoints > tolerance)) then
    SendCommands({levelTable[detailLevel][1]})
    detailLevel = detailLevel + 1
	raisePoints = 0
  end
end


local function LowerDetail()
  lowerPoints = lowerPoints + 1
  if ((detailLevel > 1) and
      (lowerPoints > tolerance)) then
    SendCommands({levelTable[detailLevel - 1][2]})
    detailLevel = detailLevel - 1
    lowerPoints = 0
  end
end


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


function widget:Initialize()
  SendCommands( { "water 3", "shadows 0", "advshading 1", "dynamicsky 0", "maxparticles 10000" } )
end


function widget:GameFrame(n)
  counter  = counter + 1
  table.insert(fpsTable, Spring.GetFPS())
  if (#fpsTable > 30 * 5) then
    table.remove(fpsTable, 1)
  end
  if ((counter+23) % ( 30 * 5 ) < 0.1) then
    counter = 0
    sum = 0
    for _, v in ipairs(fpsTable) do
      sum = sum + v
    end
    average = (sum / #fpsTable)
    if (average > maxFps) then
      RaiseDetail()
    else
      raisePoints = 0
    end
    if (average < minFps) then
      LowerDetail()
    else
      lowerPoints = 0
    end
  end
end
